package pi7codigog;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collections;
import java.util.List;
import org.antlr.v4.runtime.ANTLRFileStream;
import org.antlr.v4.runtime.BaseErrorListener;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.Parser;
import org.antlr.v4.runtime.RecognitionException;
import org.antlr.v4.runtime.Recognizer;
import org.antlr.v4.runtime.atn.ATNConfigSet;
import org.antlr.v4.runtime.dfa.DFA;
import org.antlr.v4.runtime.tree.ParseTreeWalker;
import pi7codigog.gcodeParser.CodfuncContext;
import pi7codigog.gcodeParser.CoordxContext;
import pi7codigog.gcodeParser.CoordyContext;

public class GerenciadorProgramas {

    private List<LinhaPrograma> programa = null;

    public void loadProgram(String filename) throws Exception {
        gcodeLexer lex = new gcodeLexer(new ANTLRFileStream(filename));
        ErrorListener error = new ErrorListener();
        lex.addErrorListener(error);
        CommonTokenStream tokens = new CommonTokenStream(lex);
        gcodeParser parser = new gcodeParser(tokens);
        parser.addErrorListener(error);
        gcodeParser.DefContext tree = parser.def();
        if (error.error != null) {
            throw new Exception (error.error);
        }
        if (tree == null) {
            throw new Exception("erro interno");
        }
        programa = new ArrayList<LinhaPrograma>();
        ParseTreeWalker.DEFAULT.walk(new gcodeListener(), tree);

        for (LinhaPrograma lin : programa) {
            System.out.println(lin.numeroLinha + lin.codigoG + lin.codigoM);
        }
    } //loadProgram

    class ErrorListener extends BaseErrorListener {

        String error = null;
        @Override
        public void syntaxError(Recognizer<?, ?> recognizer,
                Object offendingSymbol,
                int line,
                int charPositionInLine,
                String msg,
                RecognitionException e) {
            error = ("linha" + line + ":" + charPositionInLine +  ": " + msg);
            System.out.println(error);
        }
    } //ErrorListener

    class gcodeListener extends gcodeBaseListener {

        private String currentX = "0";
        private String currentY = "0";

        @Override
        public void enterStatement(gcodeParser.StatementContext ctx) {
            LinhaPrograma linha = new LinhaPrograma();
            linha.numeroLinha = ctx.numerolinha().num3dig().getText();
            CodfuncContext codfunc = ctx.codfunc();
            if (codfunc != null) {
                linha.codigoG = codfunc.getText();
                linha.x = getCoordX(ctx.coordx());
                linha.y = getCoordY(ctx.coordy());
            } else {
                linha.codigoM = ctx.mfunc().getText();
            }
            programa.add(linha);
        } //enterStatement

        @Override
        public void enterFimprograma(gcodeParser.FimprogramaContext ctx) {
            LinhaPrograma linha = new LinhaPrograma();
            linha.numeroLinha = ctx.numerolinha().num3dig().getText();
            linha.codigoM = ctx.mfim().getText();
            programa.add(linha);
        } //enterFimprograma

        String getCoordX(CoordxContext ctx) {
            String retval;
            if (ctx != null) {
                currentX = ctx.coord().getText();
            }
            retval = currentX;
            return retval;
        } //getCoordX

        String getCoordY(CoordyContext ctx) {
            String retval;
            if (ctx != null) {
                currentY = ctx.coord().getText();
            }
            retval = currentY;
            return retval;
        } //getCoordY
    } //gcodeListener

    public List<String> obterListagem() {
        List<String> listagem = new ArrayList<String>();
        for (LinhaPrograma p : programa) {
            String linha = "N" + p.numeroLinha + p.codigoG + p.codigoM
                    + " X" + p.x + " Y" + p.y;
            listagem.add(linha);
        } //for
        return listagem;
    } //obterListagem
} //GerenciadorProgramas
